import logging

class ImageUpload:
    __logger = logging.getLogger(__name__)

    def __init__(self, feature_extractor, milvus):
        self.feature_extractor = feature_extractor
        self.milvus = milvus

    def upload_one(self, bucket, image, key):
        self.upload_many(bucket, {image: key})

    def upload_many(self, bucket, image2keys):
        keys = []
        vectors = []
        images = []

        all_images = [image2key["image"] for image2key in image2keys]
        new_images, exist_images = self.milvus.filter_new_urls(bucket, all_images)

        print(f"总图片数: {len(all_images)}")
        print(f"新图片数: {len(new_images)}")
        print(f"已存在图片数: {len(exist_images)}")
        print(f"新图片: {new_images}")
        print(f"已存在图片: {exist_images}")

        # 将已存在的图片转换为集合，提高查找效率
        exist_images_set = set(exist_images)

        # 过滤掉已存在的图片
        filtered_image2keys = [
            image2key for image2key in image2keys
            if image2key["image"] not in exist_images_set
        ]

        if len(filtered_image2keys) == 0:
            return

        for image2key in filtered_image2keys:
            image = image2key["image"]
            key = image2key["key"]
            part = image2key["part"]
            vector = self.feature_extractor.extract_from_url(image, part)
            images.append(image)
            keys.append(key)
            vectors.append(vector)

        entities = [
            images,
            keys,
            vectors
        ]

        self.milvus.insert(bucket, entities)



